
/*
   This file is part of Valgrind, an x86 protected-mode emulator 
   designed for debugging and profiling binaries on x86-Unixes.

   Copyright (C) 2000-2002 Julian Seward 
      jseward@acm.org
      Julian_Seward@muraroa.demon.co.uk

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307, USA.

   The GNU General Public License is contained in the file LICENSE.
*/


#ifndef __VALGRIND_H
#define __VALGRIND_H


/* This file is for inclusion into client code.

   Use this macro to force the definedness of a value to be checked.
   The parameter must be an lvalue.  All bytes in the parameter are
   checked for a well-defined value, and if not, a "Use of
   uninitialised value of size 1" is emitted, regardless of the size
   of the object (illogical, yes).  

   If it happens that the lvalue is located at an invalid address, 
   you will also get complaints about "Invalid read of size 1".
*/

#define VG_CHECK_BYTES(__ptr_to_em, __num_of_em)       \
   do {                                                \
      volatile unsigned char* zzp                      \
         = (volatile unsigned char *)(__ptr_to_em);    \
      unsigned int zzsz = (unsigned int)(__num_of_em); \
      unsigned int zzi;                                \
      volatile unsigned int zzc = 0;                   \
      for (zzi = 0; zzi < zzsz; zzi++) {               \
         zzc += zzp[zzi];                              \
      }                                                \
      if (zzc == 33) fprintf(stderr,"foo from VG_CHECK_BYTES");    \
   } while (0)

#define VG_CHECK_VALUE(__lvalue)                           \
   VG_CHECK_BYTES( (volatile unsigned char *)&(__lvalue),  \
                   (unsigned int)(sizeof (__lvalue)))

#endif
